'use client';

import React, {
  type ReactNode,
  createContext,
  useCallback,
  useContext,
  useEffect,
  useRef,
  useState,
} from 'react';
import {
  getProfile,
  getSignedUrl,
  getUser,
  updateProfile as updateProfileAction,
} from '@/lib/actions';
import {
  type User,
  type Profile,
  createClient,
} from '@/utils/supabase';
import { toast } from 'sonner';

type AuthContextType = {
  user: User | null;
  profile: Profile | null;
  avatarUrl: string | null;
  isLoading: boolean;
  isAuthenticated: boolean;
  updateProfile: (data: {
    full_name?: string;
    email?: string;
    avatar_url?: string;
  }) => Promise<{ success: boolean; data?: Profile; error?: unknown }>;
  refreshUserData: () => Promise<void>;
};

const AuthContext = createContext<AuthContextType | undefined>(undefined);

export const AuthProvider = ({ children }: { children: ReactNode }) => {
  const [user, setUser] = useState<User | null>(null);
  const [profile, setProfile] = useState<Profile | null>(null);
  const [avatarUrl, setAvatarUrl] = useState<string | null>(null);
  const [isLoading, setIsLoading] = useState(true);
  const [isInitialized, setIsInitialized] = useState(false);

  const fetchingRef = useRef(false);

  const fetchUserData = useCallback(async () => {
    if (fetchingRef.current) return;
    fetchingRef.current = true;
    try {
      setIsLoading(true);
      
      const userResponse = await getUser();
      const profileResponse = await getProfile();

      if (!userResponse.success || !profileResponse.success) {
        setUser(null);
        setProfile(null);
        setAvatarUrl(null);
        return;
      }
      setUser(userResponse.data);
      setProfile(profileResponse.data);

      // Get avatar URL if available
      if (profileResponse.data.avatar_url) {
        const avatarResponse = await getSignedUrl({
          bucket: 'avatars',
          url: profileResponse.data.avatar_url,
        });

        if (avatarResponse.success) {
          setAvatarUrl(avatarResponse.data);
        } else setAvatarUrl(null);
      }

    } catch (error) {
      console.error(
        'Auth fetch error: ',
        error instanceof Error ?
          `${error.message}` :
          'Failed to load user data!'
      );
      if (!isInitialized) {
        toast.error('Failed to load user data!');
      }
    } finally {
      setIsLoading(false);
      setIsInitialized(true);
      fetchingRef.current = false;
    }
  }, [isInitialized]);

  useEffect(() => {
    const supabase = createClient();

    fetchUserData().catch((error) => {
      console.error('💥 Initial fetch error:', error);
    });

    const {
      data: { subscription },
    } = supabase.auth.onAuthStateChange(async (event, session) => {
      if (event === 'SIGNED_IN') {
        await fetchUserData();
      } else if (event === 'SIGNED_OUT') {
        setUser(null);
        setProfile(null);
        setAvatarUrl(null);
        setIsLoading(false);
      }
    });
    
    return () => {
      subscription.unsubscribe();
    };
  }, [fetchUserData]);

  const updateProfile = async (data: {
    full_name?: string;
    email?: string;
    avatar_url?: string;
  }) => {
    try {
      const result = await updateProfileAction(data);
      if (!result.success) {
        throw new Error(result.error ?? 'Failed to update profile');
      }
      
      setProfile(result.data);
      
      // If avatar was updated, refresh the avatar URL
      if (data.avatar_url && result.data.avatar_url) {
        const avatarResponse = await getSignedUrl({
          bucket: 'avatars',
          url: result.data.avatar_url,
          transform: { width: 128, height: 128 },
        });
        
        if (avatarResponse.success) {
          setAvatarUrl(avatarResponse.data);
        }
      }
      
      toast.success('Profile updated successfully!');
      return { success: true, data: result.data };
    } catch (error) {
      console.error('Error updating profile:', error);
      toast.error(error instanceof Error ? error.message : 'Failed to update profile');
      return { success: false, error };
    } finally {
      setIsLoading(false);
    }
  };

  const refreshUserData = async () => {
    console.log('Manual refresh triggered');
    await fetchUserData();
  };

  const value = {
    user,
    profile,
    avatarUrl,
    isLoading,
    isAuthenticated: !!user,
    updateProfile,
    refreshUserData,
  };

  return <AuthContext.Provider value={value}>{children}</AuthContext.Provider>;
}

export const useAuth = () => {
  const context = useContext(AuthContext);
  if (context === undefined) {
    throw new Error('useAuth must be used within an AuthProvider');
  }
  return context;
}
